#!/usr/bin/Rscript
## !/biosw/R/3.3.1/bin/Rscript

# RNA modification level estimation with pulseR
# Copyright under the MIT License

# Usage: ./run-epitranscriptomics-pipeline.R [config] <use-spike-ins>

library(pulseR)
library(yaml)
library(dplyr)

# get params and fitting options
args <- commandArgs(trailingOnly=TRUE)
# default without spike-ins
use.spikeins <- F
if (length(args)<1) {
  stop("At least configuration file must be given.\n", call.=FALSE)
} else {
  if (length(args)>1 & args[2] == 'use-spike-ins') {
    use.spikeins <- T
  }
}
params.file <- args[1]
params <- yaml::read_yaml(params.file)
opts <- params$opts
opts$use.spikeins <- use.spikeins

# set paths and save options
dirloc.output <- file.path(params$dirlocOutput, params$cell)
dir.create(dirloc.output, recursive=TRUE, showWarnings=FALSE)

opts$results.fit <- file.path(dirloc.output, "fit.rds")
opts$results.cis <- file.path(dirloc.output, "cis.rds")
saveRDS(opts, file.path(dirloc.output, "opts.rds"))

# read samples...
samples <- readRDS(file.path(params$dirlocInput, "samples.rds"))
samples <- samples[grep(params$cell, samples$cell),]
# .. and counts
counts <- readRDS(file.path(params$dirlocInput, paste("counts-", params$cell, ".rds", sep="")))
# remove input if using spike-ins, else use as reference, relevel, add formulas, create pulseData object
if (use.spikeins) {
  samples <- samples[grep('total', samples$fraction, invert=TRUE),]
  samples$fraction <- factor(samples$fraction)
  samples$fraction <- relevel(samples$fraction, ref="supernatent")
  
  counts <- counts[,grep('tot', names(counts), invert=TRUE)]
  # add ERCC spikes in
  ercc <- readRDS(file.path(params$dirlocInput, 'counts-ERCC.rds'))
  ercc <- ercc[, names(ercc) %in% names(counts)]
  counts <- rbind(counts, ercc)
  # filter lowly abundant genes (incl. ERCC spike ins)
  high.exp <- rowMeans(counts) > params$expressionThreshold
  counts <- counts[high.exp,]
  # get list of remaining spike ins and construct spike ins list
  erccList <- rownames(counts[grep('ERCC', rownames(counts)),])

  spikeIns <- list() 
  spikeIns$refGroup <- "supernatent"
  spikeList <- list()
  spikeList$supernatent <- list(erccList)
  spikeList$eluate <- list(erccList)
  spikeIns$spikeLists <- spikeList
  
  # parametrise using logit
  formulas <- MeanFormulas(
    modified    = exp(y + mu) / (1 + exp(y)),
    nonmodified = exp(mu) / (1 + exp(y))
  )
  formulas.idx <- list(
    "eluate"      = "modified",
    "supernatent" = "nonmodified"
  )
  pd <- PulseData(counts=counts,
                  conditions=samples[,"fraction", drop=FALSE], 
                  formulas=formulas, 
                  formulaIndexes=formulas.idx,
                  spikeins=spikeIns)
} else {
  samples$fraction <- factor(samples$fraction)
  samples$fraction <- relevel(samples$fraction, ref="total")
  # filter lowly abundant genes
  high.exp <- rowMeans(counts[, grep("tot", names(counts))]) > params$expressionThreshold
  counts <- counts[high.exp,]
  # parametrise using logit
  formulas <- MeanFormulas(
    total       = exp(mu),
    modified    = exp(y + mu) / (1 + exp(y)),
    nonmodified = exp(mu) / (1 + exp(y))
  )
  formulas.idx <- list(
    "total"       = "total",
    "eluate"      = "modified",
    "supernatent" = "nonmodified"
  )
  # pulseData object
  pd <- PulseData(counts=counts,
                  conditions=samples[,"fraction", drop=FALSE], 
                  formulas=formulas, 
                  formulaIndexes=formulas.idx,
                  groups=~fraction)
}
# write to disk                
saveRDS(pd, file.path(dirloc.output, "pd.rds"))

# set initial values
pars <- initParameters(list(), c("mu", "y"), pd, params$opts)
pars$mu <- log(pd$counts[,1]+1)
pars$y[] <- 0
pars$size <- 10
if (!use.spikeins) {
  for(i in seq_along(pars$normFactors)) {
    pars$normFactors[[i]][1] <- 1
  }
}

# fit model and estimate methylation levels in each fraction
result <- fitModel(pd, pars, params$opts)
saveRDS(result, opts$results.fit)

# calculate confidence intervals
cis <- list()
for(parName in c("mu", "y")) {
  cis[[parName]] <- ciGene(
    parName=parName,
    geneIndexes=seq_along(pd$counts[,1]),
    pd=pd,
    par=result,
    options=params$opts)
}
saveRDS(cis, opts$results.cis)
